# Style guide

This document describes the best practices for writing documentation at Teklia.

[[_TOC_]]

## Using this style guide

This section briefly describes common usage of the style guide for all the
involved parties.

### Writers

On your first time, read carefully over this guide to grasp its main intents.
Remembering everything by heart is not necessary.

Write a first draft of your documentation, referring to the guide again as
necessary, and create a merge request. Reviewers will then add comments to
point out possible deviations from the guide, from which you can make new
corrections, until the documentation gets merged.

Applying the style guide can feel daunting at first, but over time, with
practice, it can become ingrained; try to be patient.

### Reviewers

With code, linters and unit tests can often automatically notice most of the
issues. With text, this becomes more complex. Some rules such as enforcing
alt-text on images can be applied automatically, but others like avoiding humor
require human intervention.

Try to be strict with the style guide. When pointing out a style issue,
try to link to the corresponding section of the style guide, to help a writer
learn about it. Do not hesitate to ask questions to the writer in merge request
comments.

### Relevant discussions

Communication relevant to this style guide should be done publicly on the
`#documentation` Slack channel. Reporting issues or asking for questions on
this style guide is encouraged there.

Any change made to this guide should be widely shared to avoid situations
where some writers still use the old rules.

## File formats

### Text files

* [CommonMark](https://spec.commonmark.org/) (a common Markdown specification)
  should be used as the formatting syntax.

* Platform-specific extensions such as
  [GitHub Flavored Markdown](https://github.github.com/gfm/) or
  [GitLab Flavored Markdown](https://docs.gitlab.com/ee/user/markdown.html) can
  be used, although the consequences of potentially tying the text files to a
  particular platform should be carefully considered.

* All text files should be encoded using Unicode.

* All text files should use LF (`U+0010`) line endings.

* Whenever possible, lines in text files should not exceed 80 characters.

### Binary files

Large binary files (> 2Mb) must either be stored using
[Git LFS](https://git-lfs.github.com/) or an external service such as S3, to
avoid unnecessary overhead for Git.

### Media

The resolution, bitrate, or file format of media should be carefully chosen to
be adapted to its use within the documentation:

* Is a 5000×5000 resolution truly useful when the image will be displayed
  as 200×200?

* Is a lossless compression format such as PNG required when pixel-perfect
  precision is not?

* Is this rather uncommon and unsupported format (such as JPEG2000) required
  when a much more common format (JPEG) is available?

Some documentation generators can help with media compression automatically,
especially for images, but manual intervention can still be more efficient.

Saving bandwidth helps with SEO, as a page's size counts in search rankings,
and enhances mobile navigation and accessibility.

## Language

### Write in American English

You can refer to this long but exhaustive
[comparison of British and American English](https://en.wikipedia.org/wiki/Comparison_of_American_and_British_English).
The most well-known difference is the use of `-or` instead of `-our`, for
example with `color` and `behavior`.

### Talk to the user

Use of the *you* pronoun is encouraged:

> Arkindex allows *you* to create *your* own workers.
> To create a worker, use *your* API token to …

### Avoid offending the user

* No racist, sexist or injurious language.

* Do not assume the user's gender.
  When not referring to the user, use *they* as both singular and plural,
  instead of expressions like *he/she* or *(s)he*.

  > When a person browses a folder, they can see files …

* Avoid subjective language (`just`, `simply`, …)
  A user that tried and failed a process 10 times will not be happy to see
  how it "just" needs those "simple" steps.

### Avoid humor

Humor makes documentation harder to translate and does not add any meaningful
content.

### Avoid personal opinions

Documentation is not the place to judge of the usefulness of a feature.

### Do not mention the project's future

Saying that this feature is "Coming Soon™" can be wrong and belongs more to
blogs and marketing announcements than to documentation.

### Avoid undefined jargon

* Try to use common words where possible.

* Define specific terms in a glossary, in topical pages such as
  *Understanding Recursion*, or in the document itself.

* Avoid abbreviations such as `i18n`.

### Use passive voice for explanations

> The git commit is pulled by Arkindex. A process is started…

### Avoid long paragraphs

Long paragraphs often are the first elements to be skipped by a user skimming
through a document, as they will pay more attention to code examples, images,
lists and shorter sentences.

### Break long sentences in shorter sentences or lists

> A machine learning worker is a Docker image embedding any script, generally
> Python code, that gets automatically built upon pushing changes to a GitLab
> repository and can then be used to process elements and produce other
> elements, transcriptions, classifications, entities, metadata, etc.

This sentence can be broken down in both multiple sentences and a list:

> A machine learning worker is a Docker image. It embeds any script, generally
> Python code. The image is automatically built when pushing changes to a
> GitLab repository. It can then be used to process elements, and produce
> results such as:
>
> * Other elements
> * Transcriptions
> * Classifications
> * Entities 
> * Metadata

Shorter sentences have a grammar that is easier to parse, and lists are more
visible and concise.

### Capitalization guidelines

* Capitalize the initial letter of the first word of a sentence.

* Capitalize the initial letter of the first word of a header.

* Capitalize the initial letter of the first word of a list item.

* Capitalize the initial letter of a key: `Shift`, `Esc`, `Ctrl`.

* Capitalize all letters of acronyms, except for well-known exceptions (`SaaS`).

* Do not change capitalization of proper nouns, brand names, software names:
  `GitLab`, `Git`, `GVim`, `iCal`

* Do not capitalize to emphasize words.

* Do not capitalize compound terms followed by acronyms: `post-CI`.

* Avoid title case for headers (Capitalizing Nearly Every Word in a Title).

* Avoid starting a sentence with a word that has a lowercase initial letter,
  such as a command:

  "`apt` requires superuser privileges" should be replaced with
  "The `apt` command requires superuser privileges".

### Punctuation guidelines

* Do not add spaces before punctuation marks: `a? b: c; d, e. f—g`.

* Ignore punctuation marks from quotations, and add the relevant ones after
  quotation marks:

  `The text box should now display "To be or not to be?".`

* Do not use semicolons and avoid using en-dashes or em-dashes.
  Use paragraph breaks, split up into multiple sentences,
  or use a definition list instead.

* Use parentheses only to describe abbreviations or acronyms.

* Do not use quotation marks to denote ambiguous terms.
  Define the term or use another term.

## Basic formatting blocks

### Emphasis

* Avoid *emphasis* as it is often rarely visible.

  * Prefer quotation marks or blockquotes for quoting.

  * Do not use emphasis to point out ambiguous terms.
    Use other terms instead, or define them clearly.

* Use **strong emphasis** for interface items:

  > Click on the **IIIF manifest** button.

* Prefer asterisks (`*` and `**`) over underscores (`_` and `__`) to use an
  emphasis in Markdown.

### Blockquotes

Use blockquotes exclusively for verbatim quotes or warnings.

### Code blocks

Use code blocks to represent blocks of code or terminal output.

Code blocks should be used carefully as they can attract the attention of users,
especially developers, as they can copy-paste without reading the surrounding
text.

Prefer three backticks (```) over three tildes (~~~) to introduce code blocks,
and do not use four-space indentations.

When possible, relevant languages should be added to code blocks to enable
syntax highlighting on compatible generators:

~~~markdown
```python
raise ZeroDivisionError
```
~~~

Inline code blocks should only be used for very short code blocks that do not
hinder reading a sentence, such as short commands (`git push`) or programming
language keywords (`True`, `False`).

### Keyboard shortcuts

Represent keyboard keys and shortcuts using the `<kbd>` HTML tag, one for each
key. When multiple keys should be pressed together, combine them with a plus
sign:

> Press <kbd>Ctrl</kbd> + <kbd>C</kbd> to interrupt the script.

## Headers

### Header levels

* One and only one first-level header should be included at the top of the page.

  * The first-level header may be omitted if it is included by a static site
    generator.

* Increment header levels one at a time:

  ```markdown
  # First level

  ## Second level <!-- good -->

  #### Fourth level <!-- bad: third level first -->
  ```

* Do not go below fourth-level headers.
  This likely means the document should be split into multiple documents or
  restructured.

* In how-tos, use second-level headings for general concepts:
  * Preparing your machine learning worker
  * Setting up your environment
  * Uploading your data

* In how-tos, use third-level headings for specific concepts:
  * Creating a Git repository
  * Installing virtualenvwrapper
  * Retrieving your API token

* In how-tos, use fourth-level headings for procedures:
  * To create a Git repository
  * To install virtualenvwrapper
  * To retrieve your API token

#### Example

```markdown
# Creating a machine learning worker

## Preparing your environment

This section will guide you through setting up the various development tools to
write a Python machine learning worker and push it to Arkindex.

### Installing virtualenvwrapper

`virtualenvwrapper` is a tool that makes managing Python virtual environments
simpler using short commands such as `workon`.

#### To install virtualenvwrapper

1. Open a terminal.
2. Run `pip3 install virtualenvwrapper`
3. Add the following to your `.bashrc` file:
   ...
```

### Avoid dead zones

There should always be some text between two titles.

```markdown
## This is some title

<!-- dead zone: no text in between titles -->

### This is some subtitle
```

## WIIFM

*WIIFM* is technical writing jargon for "What's in it for me?", or "What's in
it for the user?". A WIIFM is a single paragraph below a section title that
briefly describes what the section is about.

This helps users orient themselves, as they will first look at the header,
then look at this paragraph to tell if this is the portion of the page they
want to read.

In how-tos, add a WIIFM for each of the general and specific concept headers.

If a concept is too long or too complex to be described in a single paragraph,
try to summarize it as best as you can, then on another paragraph, add a link
to a page that gives a more in-depth description of the concept:

```markdown
For more information, see [Understanding Elements](understanding-elements.md)
```

## Procedures

Procedures are the steps the user will actually follow; a recipe's
instructions section.

### Use imperative verbs

Start a procedure with an imperative verb. Avoid using "To" as this is reserved
for procedure section headings:

```markdown
#### To create a Git repository

1. Open a terminal
2. Run `git init`
```

### Avoid going over 9 steps

This is known as the *rule of 9s* in technical writing: too many steps often
mean those should be split into multiple sections.

For example, instead of 50 steps to download, install, configure then learn to
use a piece of software, break them up into separate sections on downloading,
installing, configuring, and each of the different actions that can be made
with this software.

### Avoid optional steps

Optional steps should be added as tips or notes, or split up into different
sections.

For example, instead of having steps depending on Python 2 or 3, you can use
two headers:

```markdown
#### To create a worker with Python 2

1. Do …

#### To create a worker with Python 3

1. Do …
```

## Images

### Add a short alternate text

Markdown's image tag helpfully has an alt-text portion just for this purpose:

```markdown
![alt text](url)
```

Adding alt text helps with SEO, accessibility, and can sometimes even help
people with normal vision by describing images.

### Use figure and figcaption

Using `<figcaption>` instead of a regular paragraph to give a short caption to
an image also helps with SEO and accessibility, as the caption becomes
semantically attached to the image. This also makes styling captions easier
with CSS.

```html
<figure>
  <img src="..." alt="..." />
  <figcaption>Status page for a running process</figcaption>
</figure>
```

### Assume images may not be seen

While including an image can help with understanding a set of steps, locating
an interface item, or checking that a procedure did give the correct result,
visually impaired people, or users with very low bandwidths may not see any
media that is not purely text.

Some images can be too hard to describe purely using alternate text and
captions.

As you are writing 

## Lists

### Use lists for enumerations

Instead of enumerating items in a sentence using "or" or "and":

> It produces elements, transcriptions, classifications and entities.

Use an unordered list, introduced by a sentence ending in a colon:

> It produces the following result types:
>
> * elements
> * transcriptions
> * classifications
> * entities

When an item in a list is a complete sentence, end it with a period:

> This process can fail when one of the conditions are met:
>
> * A network error occurs.
> * A power outage occurs.
> * The Stop button is clicked.

### Use numbered lists for ordered items

Numbered lists should only be used when the order of the items matter; usually
for a procedure.

If you need to list 5 things, but the order does not matter, it is better to
use a regular bulleted list; the numbers do not give any meaningful information
to the user.

### Add line breaks for long list items

The CommonMark specification gives meaning to
[line breaks between list items](https://spec.commonmark.org/0.29/#loose):
a single line break between two items of the same list causes all items to have
paragraph tags (`<p>`).

This adds extra spacing to list items and should be used to enhance readability
for lists with long items, sublists, or other content such as code blocks.

### Use definition lists to describe multiple items

To describe the various attributes of an object, or various interface items,
you can use an HTML definition list instead of a bulleted list:

```markdown
<dl>
  <dt>name</dt>
  <dd>Name of the object.</dd>
  <dt>type</dt>
  <dd>Type of the object.</dd>
</dl>
```

<dl>
  <dt>name</dt>
  <dd>Name of the object.</dd>
  <dt>type</dt>
  <dd>Type of the object.</dd>
</dl>

Accessibility tools can navigate more easily in definition lists than regular
lists.

Definition lists are available in some flavors of Markdown such as
PHP Markdown Extra and Python-Markdown, but do not exist in pure Markdown.
Using the regular `<dl>`, `<dt>` and `<dd>` tags is therefore recommended.

## Tables

### Prefer other representations

* Avoid tables with large amounts of text.

* Replace single-column tables with lists.

* Replace tables with key-value pairs, such as descriptions of a list of files,
  with definition lists.

### Enhance accessibility

Tables are one of the most difficult HTML elements to make accessible.
Adding the `scope="col"` attribute to a `<table>` element can solve many
issues but might not be supported by the documentation generator.

Similarly to images, you can wrap tables into HTML `figure` tags and use
`figcaption` to describe them. This can make tables easier to reference in
other parts of the document and easier to discover for screen readers.

## Appendixes

When some information could be useful as a quick reference, for example a table
that says which file formats are supported in which processes, or when a guide
needs to refer to a figure, table or list from another guide, consider doing
one of the following:

* Put this element under a separate header to make it visible from a table of
  contents and allow it to be linked using an anchor.

  * Images and tables can be put in `figure` elements with an `id` attribute
    instead of a header.

* Put this element in a separate document and always refer to it using links.

## To-dos

To-do items intended for writers, for example to fill in incomplete sections,
must follow these guidelines:

* Always use HTML comments `<!-- -->` to prevent the to-do items or other
  writer-only notes from being included in the published docs.

* Prefix your to-dos with `TODO:` to make them easier to find.

* Avoid `TODO` without a colon (`:`) as some IDEs, editor extensions, etc.
  might require a colon.

* Avoid `<!-- TODO: -->` without an explicit message on what there is to do.

* If a to-do item is not to be completed before the documentation gets merged
  or published, consider creating a GitLab issue.

* Add links to relevant GitLab issues in your to-do comments.

An example of a to-do comment with a GitLab issue follows:

```markdown
<!--
TODO: Update screenshots for v1.0
https://gitlab.teklia.com/arkindex/doc/issues/99999
-->
```

The [Todo Tree](https://marketplace.visualstudio.com/items?itemName=Gruntfuggly.todo-tree)
extension is recommended for Visual Studio Code users.

## Project-specific guidelines

This section describes guidelines that are purely specific to the Arkindex
documentation; if this guide was to be copied for a different project,
notably one using a different static site generator or different templates,
those guidelines do not apply.

### Front matter

To learn more about front matter, see the Zola documentation for
[front matter in pages](https://www.getzola.org/documentation/content/page/#front-matter)
and [front matter in sections](https://www.getzola.org/documentation/content/section/#front-matter).

* Always include front matter in all Markdown files.

* Always include a `title` in all front matters.

  * The `title` replaces the usual first-level Markdown header (`#`).

* Always include a `weight` in all front matters.

  * A new `weight` value should be the next available multiple of 10.

    For example, if you had a page with weight 10 and one with weight 12,
    the next page should have weight 20.

  * If you run out of integer weights to insert a page in between two
    pages, shift all of the other weights to become multiples of 10.

* Prefer `draft = true` over `render = false` for draft pages.

* For section indexes (`_index.md`), also include the following:

  ```toml
  sort_by = "weight"
  insert_anchor_links = "right"
  ```

  > **Warning:** Sorting by weight means that pages without a weight will not
  > be included in the HTML output.

* Do not put any page outside of a section. Doing so can cause a strange error
  with the current theme:

  ```
  Section `` not found
  ```

  For more information, see
  [this GitHub issue](https://github.com/getzola/zola/issues/641).

<!--
TODO: Arkindex terminology guide, e.g. prefer "item" over "element" when it's
not an Arkindex element
-->

## Warnings

Warnings use a [Zola shortcode](https://www.getzola.org/documentation/content/shortcodes/) to
display as a visible notification to the end user. 

```html
{% warning() %}
You can write any Markdown here, it will be processed and displayed in a nice
orange box.
{% end %}
```

Use warnings for important information that the user should read and apply
carefully to prevent injury or damage.

Using a warning to document a common pitfall might mean the section could be
rewritten, possibly with more detail or more clarity, to better emphasize on
the issue without the need for a warning.

## Figures

You can use another shortcode to display figures, with an optional caption.

```
{{ figure(image="path/to/image.png", caption="My nice image", height=300) }}
```

## Recommended reading

* [Documentation Principles](https://www.writethedocs.org/guide/writing/docs-principles/)
* [GNOME Documentation Style Guide](https://developer.gnome.org/gdp-style-guide/2.32/gdp-style-guide.html)
* [Dealing with Knowledge Gaps](https://www.writethedocs.org/guide/imposter/#dealing-with-knowledge-gaps)
  when writing documentation for something you do not know much about.
